// Copyright  2008 for Adam Tibi , www.AdamTibi.NET . All Rights Reserved
using System;
using System.Web.UI;
using System.ComponentModel;
using System.Web;
using System.Security.Permissions;
using System.Web.UI.WebControls;

namespace AT.Web.UI.Controls {

    /// <summary>
    /// Web control to keep the session alive by refreshing every set amount of time
    /// </summary>
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [ToolboxData("<{0}:SessionKeepAlive runat=server />")]
    public class SessionKeepAlive : WebControl {

        public SessionKeepAlive() : base(HtmlTextWriterTag.Img) {
        }

        /// <summary>
        /// Set, in minutes, the duration to trigger the session. The default value is one minute
        /// </summary>
        [Category("Behavior")]
        [DefaultValue(1)]
        public int Duration {
            get {
                return (int)(ViewState["Duration"] ??  1);
            }
            set {
                ViewState["Duration"] = value;
            }
        }

        /// <summary>
        /// The path of the image's URL that will be requested from the server, the default is ~/session-keep-alive.ashx
        /// </summary>
        [Category("Behavior")]
        [DefaultValue("~/session-keep-alive.ashx")]
        public string PageUrl {
            get {
                return (string)(ViewState["PageUrl"] ?? "~/session-keep-alive.ashx");
            }
            set {
                ViewState["PageUrl"] = value;
            }
        }

        /// <summary>
        /// Applies in-line CSS style to hide the generated image. The default is true.
        /// Alternatively, you can set this to false and hide the image using your own
        /// CSS to keep your page XHTML Strict compatible.
        /// </summary>
        [Category("Behavior")]
        [DefaultValue(true)]
        public bool AutoHide {
            get {
                return (bool)(ViewState["AutoHide"] ?? true);
            }
            set {
                ViewState["AutoHide"] = value;
            }
        }

        protected override void OnPreRender(EventArgs e) {
            base.OnPreRender(e);

            if (AutoHide) {
                this.Style.Add(HtmlTextWriterStyle.Display, "none");
            }

            Page.ClientScript.RegisterClientScriptResource(typeof(SessionKeepAlive),
                "AT.Web.UI.Controls.JavaScript.js");

            Page.ClientScript.RegisterStartupScript(typeof(SessionKeepAlive), "SessionKeepAlive",
                string.Format(@"window.setInterval('SessionKeepAlive(\'{0}\', \'{1}\');', {2} );", 
                ClientID, ResolveClientUrl(PageUrl), Duration * 60000), true);
        }


        protected override void Render(HtmlTextWriter writer) {
            writer.AddAttribute(HtmlTextWriterAttribute.Src, ResolveClientUrl(PageUrl));
            writer.AddAttribute(HtmlTextWriterAttribute.Alt, " ");
            base.RenderBeginTag(writer);
            base.RenderEndTag(writer);
        }

    }
}
